package src.gui;


import src.VisualXML;
import src.desktop.InnerFrame;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;

import org.dom4j.Element;
import org.dom4j.Attribute;

/**
 * JTree for displaying customized XML tree
 */ 
public class XMLTree extends JTree implements ActionListener
{
   JPopupMenu popup; // popupmenu
   VisualXML parent;
   ExtendedTreeNode root;
   
   private Icon customOpenIcon = new ImageIcon("images/record_22.gif");
   private Icon customClosedIcon = new ImageIcon("images/record_21.gif");
   private Icon customLeafIcon = new ImageIcon("images/textNode.gif");
  
   /** Constructor */
   public XMLTree(VisualXML jf)
   {
      super();
      parent = jf;
   }
   
   /** Constructor */
   public XMLTree (ExtendedTreeNode r, VisualXML jf)
   {
      super(r);
      root = r;
      getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
      parent = jf;
      
      // Set icons 
      DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
      renderer.setOpenIcon(customOpenIcon);
      renderer.setClosedIcon(customClosedIcon);
      renderer.setLeafIcon(customLeafIcon);
      setCellRenderer(renderer);
    
      // define the popup
      popup = new JPopupMenu();
      JMenuItem mi = new JMenuItem(parent.getString("XMLTree.popupMenu_show"));
      mi.addActionListener(this);
      mi.setActionCommand("show");
      popup.add(mi);
      popup.addSeparator();
      mi = new JMenuItem(parent.getString("XMLTree.popupMenu_cloneNode"));
      mi.addActionListener(this);
      mi.setActionCommand("clone");
      popup.add(mi); 
      mi = new JMenuItem(parent.getString("XMLTree.popupMenu_insertNode"));
      mi.addActionListener(this);
      mi.setActionCommand("insert");
      popup.add(mi);  
      popup.addSeparator();  
      mi = new JMenuItem(parent.getString("XMLTree.popupMenu_remove"));
      mi.addActionListener(this);
      mi.setActionCommand("remove");
      popup.add(mi);  
      popup.setOpaque(true);
      popup.setLightWeightPopupEnabled(true);

      addMouseListener
      (
         new MouseAdapter ()
         {
            public void mouseReleased( MouseEvent e )
            {
               if ( e.isPopupTrigger())
               {
                  try
                  {// if a node clicked, show popup 
                     ((DefaultMutableTreeNode) getPathForLocation(e.getX(),e.getY())
                         .getLastPathComponent()).getUserObject();
                     // set clicked node to be selected
                     setSelectionPath(getPathForLocation(e.getX(),e.getY()));
                     popup.show( (JComponent)e.getSource(), e.getX(), e.getY());
                  }
                  catch (NullPointerException ex)
                  {// User did not click on a node
                  }
               }
            }
         }
      );
   }
 
   /**
    * Recursively creates tree structure
    */       
   public void createNodes(ExtendedTreeNode DMTNRoot, Element ERoot)
   {
      Iterator elementIterator = ERoot.elementIterator();
      while(elementIterator.hasNext())
      {
         Element element = (Element)elementIterator.next();
         ExtendedTreeNode node = new ExtendedTreeNode(element, treeNodeLabel(element));
         DMTNRoot.add(node);
         
         if (element.nodeCount() > 0)
         {
            createNodes(node,element);
         }
      }
   }
   
   public void actionPerformed(ActionEvent ae)
   {
      ExtendedTreeNode dmtn, node;

      // get selected node
      TreePath path = this.getSelectionPath();
      dmtn = (ExtendedTreeNode) path.getLastPathComponent();
      if (ae.getActionCommand().equals("show"))
      {
         //System.out.println(dmtn.getUserObject());
         InnerFrame iframe = ((VisualXML)parent).createFrame(dmtn.getContent(),dmtn.getContent().getName(),null, false);
         if (iframe != null)
         {
             Point loc = ((VisualXML)parent).findFreePlace();
             iframe.setLocation(loc.x, loc.y);
         }
      }
      else if (ae.getActionCommand().equals("remove"))
      {
         node = (ExtendedTreeNode)dmtn.getParent();
         Element e = dmtn.getContent();
         Element eParent = (Element) node.getContent();
         eParent.remove(e);
         dmtn.removeAllChildren();
         node.remove(dmtn);
         ((DefaultTreeModel )this.getModel())
           .nodeStructureChanged((TreeNode)node); 
           ((VisualXML)parent).setEdited(true);
           
         parent.updateIFrames(e);
    }
    else if (ae.getActionCommand().equals("insert"))
    {
       String s = (String)JOptionPane.showInputDialog(parent,
                                        ((VisualXML)parent).getString("XMLTree.dialog_enterName"),
                                        ((VisualXML)parent).getString("XMLTree.dialog_enterName_title"),
                                        JOptionPane.PLAIN_MESSAGE,null,null,"");

       // String returned 
       if ((s != null) && (s.length() > 0))
       {
          Element e = ((ExtendedTreeNode)dmtn).getContent().addElement(s);
          node = new ExtendedTreeNode(e, s);
          dmtn.add(node);
          ((VisualXML)parent).setEdited(true);
          // Refreshing the tree struct
          ((DefaultTreeModel )this.getModel()).nodeStructureChanged((TreeNode)dmtn);
          parent.updateIFrames(null);
       }
    }
    else if (ae.getActionCommand().equals("clone"))
    {
       cloneNode(dmtn);
       ((VisualXML)parent).setEdited(true);
       // Refresh iframes
       parent.updateIFrames(null);
    }
  }
  
   /** 
    * Deep clone of a node
    */
   public void cloneNode(ExtendedTreeNode dmtn)
   {
      Element e = (Element) dmtn.getContent().createCopy();
      Element parent = dmtn.getContent().getParent();
      if (parent != null)
      {
         dmtn.getContent().getParent().add(e);
         ExtendedTreeNode node = new ExtendedTreeNode(e, e.getName());
         ExtendedTreeNode parentNode = (ExtendedTreeNode) dmtn.getParent();
         parentNode.add(node);
         createNodes(node, e);
         refresh(parentNode);
      }
   }
  
   /**
    * Finds node which contains given element
    */
   public ExtendedTreeNode findNode(Element elem)
   {
      for (Enumeration e = root.breadthFirstEnumeration() ; e.hasMoreElements() ;)
      {
         ExtendedTreeNode etn = (ExtendedTreeNode) e.nextElement();
         if (etn.getContent() == elem)
         {
            return etn;
         }
      }
      return null;
   }
  
   /**
    * Refreshes XMLTree
    */     
   public void refreshNode(ExtendedTreeNode node)
   {
      node.setUserObject(treeNodeLabel(node.getContent()));
      refresh(node);
   }
   
   /**
    * Removes given node from the tree and refreshes tree
    */       
   public void removeNode(ExtendedTreeNode dmtn)
   {
      ExtendedTreeNode node = (ExtendedTreeNode)dmtn.getParent();
      dmtn.removeAllChildren();
      node.remove(dmtn);
      refresh(node);
  }

   /**
    * Renames node and refreshes tree
    * 
    * @param node node to be renamed
    * @param s new name            
    */       
   public void renameNode(ExtendedTreeNode node, String s)
   {
      node.setUserObject(s);
      refresh(node);
   }
  
   /**
    * Refreshes tree
    */       
   public void refresh(ExtendedTreeNode parentNode)
   {
      ((DefaultTreeModel )this.getModel()).nodeStructureChanged((TreeNode)parentNode);
   }
   
   /**
    * Adds new node to parent ndoe and refreshes tree
    */       
   public void addNode(ExtendedTreeNode parent, Element e)
   {
      parent.add(new ExtendedTreeNode(e, e.getName()));
      refresh(parent);
   }
    
  /** If the XML element has no attributes, the JTree node
   * will just have the name of the XML element. If the
   * XML element has attributes, the names and values of the
   * attributes will be listed in parens after the XML
   * element name. For example:
   * XML Element: <bla>
   * JTree Node:  bla
   * XML Element: <bla foo="bar" baz="quux">
   * JTree Node:  bla (foo=bar, baz=quux)
   */
   private static String treeNodeLabel(Element e)
   {
      String treeNodeLabel = e.getName();
      // IF ANY ATTRIBUTES -> PRINT THEM TOO 
      if (e.attributeCount()>0)
      {
         Iterator attrI = e.attributeIterator();
         int i = 0;
         treeNodeLabel += " (";
         while (attrI.hasNext())
         {// attach to the end of element's name
            Attribute attr = (Attribute) attrI.next();
            if (i > 0)
            {
               treeNodeLabel = treeNodeLabel + ", ";
            }
            treeNodeLabel = treeNodeLabel+attr.getName()+"="+attr.getValue();
            i++;
         }
         treeNodeLabel = treeNodeLabel + ")";
      }
      return  treeNodeLabel;
   }
}
