package src;
import desktop.*;
import themes.*;
import gui.*;
import xml.*;
import desktop

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;

import java.util.Locale; 
import java.util.ResourceBundle;
import java.util.Iterator;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.DocumentHelper;
import org.dom4j.DocumentException;

import javax.swing.tree.*;
import javax.swing.plaf.metal.*;

/**
 * Main class, extends JFrame and contains XMLTree, Menu, and DesktopPane
 */ 
public class VisualXML extends JFrame implements ActionListener
{
   protected File xmlFile;                // opened xml file
   protected boolean newFile = false;
   protected static Document doc;         // DOM representation of xml doc
   public XMLTree tree;                   // visual representation of DOM doc :)
   protected JScrollPane treeView;        // scrollpane for XMLtree
   protected Container treeViewContainer; // tree view container
   protected JSplitPane content;
   protected JScrollPane scrollDesktop;   // scrollpane for desktop
   
   protected JPopupMenu desktopViewPopup;
   protected DesktopView deskView;
   
   public MDIDesktopPane desktop;
   protected Locale currentLocale;
   protected ResourceBundle messages;
   
   protected boolean edited; // dirty bit
   protected JToolBar toolBar = new JToolBar();
   
   /**
    * Constructor
    * 
    * @param language language of localization bundle
    * @param country country             
    */           
   public VisualXML(String language, String country)
   {
      super();

      try
      {
         currentLocale = new Locale(language, country);
         messages = ResourceBundle.getBundle("lang/MessagesBundle", currentLocale);
      }
      catch (Exception e)
      {
         System.out.println("Couldn't find locale messages bundle");
         System.exit(0);
      }
      
      setTitle(messages.getString("VisualXML.appname"));

      //Make the big window be indented 50 pixels from each edge of the screen.
      int inset = 50;
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      setBounds(inset, inset, screenSize.width  - inset*2, screenSize.height - inset*2);

      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

      //Set up the GUI.
      content = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT); //a specialized layered pane

      try
      {
         String laf = UIManager.getCrossPlatformLookAndFeelClassName();
         UIManager.setLookAndFeel(laf);
         MetalLookAndFeel.setCurrentTheme(new ContrastTheme());
         UIManager.put("swing.boldMetal", Boolean.FALSE);
      } catch (Exception e)
      {
         System.err.println("Couldn't use system look and feel.");
      }
      /*try
          {     UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
            } catch (Exception e) {
                System.err.println("Couldn't use system look and feel.");
            }
       */

      edited = false;
      
      //Create XMLTree
      tree = new XMLTree(this);
  
      //Create the scroll pane and add the tree to it. 
      treeView = new JScrollPane(tree);
      treeViewContainer = tree.getParent();
      treeViewContainer.setBackground(new Color(245,245,245));
    
      desktop = new MDIDesktopPane();

      scrollDesktop = new JScrollPane(desktop);
      scrollDesktop.getVerticalScrollBar().setUnitIncrement(10);

      //Add the scroll panes to a split pane.
      content.setTopComponent(treeView);
      content.setBottomComponent(scrollDesktop);

      Dimension minimumSize = new Dimension(150, 50);
      treeView.setMinimumSize(minimumSize);
      treeViewContainer.remove(tree);
      getContentPane().add(createToolBar(),BorderLayout.PAGE_START);
      getContentPane().add(content);
      
      setJMenuBar(createMenuBar());
      setVisible(true);

      //Make dragging a "LIVE"
      desktop.setDragMode(JDesktopPane.LIVE_DRAG_MODE);
      desktopViewPopup = new JPopupMenu();
      desktopViewPopup.setBorder(BorderFactory.createLineBorder(Color.darkGray, 2));
      deskView = new DesktopView(this);
      desktopViewPopup.add(deskView);
   }
 
   /**
    * Creates menubar
    */       
   protected JMenuBar createMenuBar()
   {
      JMenuBar menuBar = new JMenuBar();
      //Set up the lone menu.
      JMenu menu = new JMenu(messages.getString("VisualXML.menuDoc"));
      menuBar.add(menu);
      //Set up the first menu item.
      JMenuItem menuItem = new JMenuItem(messages.getString("VisualXML.menuDocNew"), new ImageIcon("images/new_small.png"));
      menuItem.setMnemonic(KeyEvent.VK_N);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("new");
      menuItem.addActionListener(this);
      menu.add(menuItem);
      //Set up the first menu item.
      menuItem = new JMenuItem(messages.getString("VisualXML.menuDocOpen"), new ImageIcon("images/open_small.png"));
      menuItem.setMnemonic(KeyEvent.VK_O);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("open");
      menuItem.addActionListener(this);
      menu.add(menuItem);
      menu.addSeparator();
      //Set up the first menu item.
      menuItem = new JMenuItem(messages.getString("VisualXML.menuDocSave"), new ImageIcon("images/save_small.png"));
      menuItem.setMnemonic(KeyEvent.VK_S);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("save");
      menuItem.addActionListener(this);
      menu.add(menuItem);

      //Set up the first menu item.
      menuItem = new JMenuItem(messages.getString("VisualXML.menuDocSaveAs"));
      menuItem.setActionCommand("saveas");
      menuItem.addActionListener(this);
      menu.add(menuItem);
      menu.addSeparator();        
      //Set up the first menu item.
      menuItem = new JMenuItem(messages.getString("VisualXML.menuDocClose"));
      menuItem.setMnemonic(KeyEvent.VK_C);
      menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
      menuItem.setActionCommand("close");
      menuItem.addActionListener(this);
      menu.add(menuItem);

      menu.addSeparator();

      //Set up the second menu item.
      menuItem = new JMenuItem(messages.getString("VisualXML.menuDocQuit"));
      menuItem.setActionCommand("quit");
      menuItem.addActionListener(this);
      menu.add(menuItem);

      menu = new JMenu(messages.getString("VisualXML.menuHelp"));
      menuBar.add(menu);

      menuItem = new JMenuItem(messages.getString("VisualXML.menuDocAbout"));
      menuItem.setActionCommand("about");
      menuItem.addActionListener(this);
      menu.add(menuItem);
      
      return menuBar;
   }
   
   /**
    * Creates toolbar
    */       
   public JToolBar createToolBar ()
   {
      String PATH_WORKING = System.getProperty("user.dir");
      String PATH_ICONS = "/images/";
      ImageIcon icon;
        
      int counter = 0;
      String imageName;
      String[] actionCommand = {"new", "open", "save", "settings", "root", "desktop-view"};
      String[] toolTipText = {getString("VisualXML.toolbar_newFile"),getString("VisualXML.toolbar_openFile"),
         getString("VisualXML.toolbar_save"),getString("VisualXML.toolbar_settings"),
         getString("VisualXML.toolbar_showRoot"), getString("VisualXML.toolbar_desktopView")};

      toolBar.setFloatable(false);
      
      for (String s : actionCommand)
      {
         imageName = s + ".png";    
        
         icon = new ImageIcon(PATH_WORKING + PATH_ICONS + imageName);
         JButton button = new JButton();

         //Create and initialize the button.
         button.setActionCommand(s);
         button.setToolTipText(toolTipText[counter]);
         button.addActionListener((ActionListener) this);

         if (icon != null)
         {// image found
            button.setIcon(icon);
         }
         else
         {// no image found
            button.setText(toolTipText[counter]);
         }
          
         if (counter ==  3 || counter == 4)
         {
            toolBar.addSeparator();
         }
         if (counter == 5)
         {
            button.addMouseListener(new MouseAdapter()
            {
               public void mouseClicked(MouseEvent e)
               {
                  deskView.refresh();
                  desktopViewPopup.show(e.getComponent(),e.getX(), e.getY());
               }
            });
         }
         toolBar.add(button);
         counter++;
      }
      return toolBar;
   }
   
   /**
    * Reactions for menu selections
    */       
   public void actionPerformed(ActionEvent e)
   {
      if ("new".equals(e.getActionCommand()))
      { // new document
         if (!saveIfChanged()) return; // Canceled
         String s = (String)JOptionPane.showInputDialog(this,
                                getString("VisualXML.dialog_newDoc"),
                                getString("VisualXML.dialog_newDoc_title"),
                                JOptionPane.PLAIN_MESSAGE,null,null,"");
         // String returned 
         if ((s != null) && (s.length() > 0))
         {
            newFile = true;
            xmlFile = null;
            doc = DocumentHelper.createDocument();
            doc.setXMLEncoding("UTF-8");
            doc.addElement(s);
            resetTitle();
            edited = true;
            rebuildXMLTree();
            // close iframes of previous doc
            JInternalFrame iframes[] = desktop.getAllFrames();
            for (int i=0; i<iframes.length;i++)
            {
               desktop.remove(iframes[i]);
            }

            // create iframe of the root element 
            createFrame(doc.getRootElement(), doc.getRootElement().getName(), null);
         }

      }
      if ("open".equals(e.getActionCommand()))
      { // opens a xml file
         if (!saveIfChanged()) return; // Canceled
            
         File file = chooseFile(true);
         if (file != null)
         {
            xmlFile = file;
                  
            // process xml and create DOM tree
            ProcessXML p = new ProcessXML();
            p.createDOM(xmlFile.getPath());
            doc = p.getDOMDoc();
            if (doc == null)
            {
               JOptionPane.showMessageDialog(this, getString("VisualXML.dialog_docNotWellFormed"),getString("VisualXML.dialog_docNotWellFormed_title"),JOptionPane.ERROR_MESSAGE);
               return;
            }
                  
            resetTitle();
            edited = false;

            // rebuild XMLTree
            rebuildXMLTree();

            // close iframes of previous doc
            JInternalFrame iframes[] = desktop.getAllFrames();
            for (int i=0; i<iframes.length;i++)
            {
               desktop.remove(iframes[i]);
            }

            // create iframe of the root element 
            createFrame(doc.getRootElement(), doc.getRootElement().getName(), null);
         }
      }
      else if ("close".equals(e.getActionCommand()))
      { // closes a xml file
         if (!saveIfChanged()) return; // Canceled
         
         // close iframes of previous doc
         JInternalFrame iframes[] = desktop.getAllFrames();
         for (int i=0; i<iframes.length;i++)
         {
            desktop.remove(iframes[i]);
         }
               
         // remove XMLTree
         if (tree != null)
         {
            treeViewContainer.remove(tree);
            treeViewContainer.repaint();
         }
              
         // help gc
         doc = null;
         xmlFile = null;
         tree = null;

         edited = false;
         resetTitle();
      }
      else if ("root".equals(e.getActionCommand()))
      { // shows root element
         boolean root = false;
        
         JInternalFrame[] allFrames = desktop.getAllFrames();
         for (int i = 0; i < allFrames.length; i++)
         {
            if (((InnerFrame)allFrames[i]).getParentInnerFrame() == null)
            {// found
               root = true;
               break;
            }
         }
           
         if (!root)
         {
            if (doc != null)
            {
               createFrame(doc.getRootElement(),doc.getRootElement().getName(), null);
            }
         }
      }
      else if ("saveas".equals(e.getActionCommand()))
      { // Write XML to an other file
         if (doc != null)
         {
            File file = chooseFile(false);
            if (file != null)
            {
               xmlFile = file;
               try
               {
                  ProcessXML.saveXML(xmlFile, doc);
               }
               catch (IOException ioe)
               {
                  JOptionPane.showMessageDialog(this, getString("VisualXML.dialog_saveError"),getString("VisualXML.dialog_saveError_title"),JOptionPane.ERROR_MESSAGE);
                  return;
               }
               edited = false;
               resetTitle();
            }
         }
      }
      else if ("save".equals(e.getActionCommand()))
      { // Write XML to the same file
         save();
         resetTitle();
      }
      else if ("about".equals(e.getActionCommand()))
      { // Shows form About
         About about = new About(this, true);
         about.setLocationRelativeTo(null);//centers on screen
about.pack();
//about.show();
         about.setVisible(true);
      }
      else if ("quit".equals(e.getActionCommand()))
      { // quit
         // doc's been changed.. save?
         if (edited)
         {
            int n = JOptionPane.showConfirmDialog(this,
                    getString("VisualXML.dialog_docChanged"),
                    getString("VisualXML.dialog_docChanged_title"),
                    JOptionPane.YES_NO_CANCEL_OPTION);
            if (n == JOptionPane.YES_OPTION)
            {// save
               save();
            }
            else if (n == JOptionPane.CANCEL_OPTION)
            {// cancel
               return;
            }
         }
         quit();
      }
   }

   /**
    * Rebuilds and repaints the XMLTree
    */
   protected void rebuildXMLTree()
   {
            // rebuild XMLTree
            ExtendedTreeNode top =  new ExtendedTreeNode(doc.getRootElement(),doc.getRootElement().getName());
            // remove the old one
            if (tree != null)
            {
               treeViewContainer.remove(tree);
            }
            tree = new XMLTree(top,this);
            // create the nodes.
            tree.createNodes(top, doc.getRootElement());
            tree.setRootVisible(true);
            tree.expandRow(0);
            treeViewContainer.add(tree);
            treeViewContainer.repaint();
   }
   
   /**
    * if document is not null, saves xml doc. if it is a new doc, shows 
    * saveDialog for choosing target file
    */
   protected void save()
   {
      if (doc != null)
      {
         if (xmlFile == null)
         {
            xmlFile = chooseFile(false);
         }
         if (xmlFile != null && edited == true)
         {
            try
            {
               ProcessXML.saveXML(xmlFile, doc);
            }
            catch (IOException ioe)
            {
               JOptionPane.showMessageDialog(this, getString("VisualXML.dialog_saveError"),getString("VisualXML.dialog_saveError_title"),JOptionPane.ERROR_MESSAGE);
               return;
            }
            edited = false;
            resetTitle();
         }
         newFile = false;
      }
   }
   /**
    * If doc has been changed, shows confirm dialog and saves XMLdoc to a file
    *
    * @return false if cancel button's been clicked
    */
   protected boolean saveIfChanged()
   {
      if (doc != null)
      {// doc's been changed.. save?
         if (edited)
         {
            int n = JOptionPane.showConfirmDialog(this,
                    getString("VisualXML.dialog_docChanged"),
                    getString("VisualXML.dialog_docChanged_title"),
                    JOptionPane.YES_NO_CANCEL_OPTION);
            if (n == JOptionPane.YES_OPTION)
            {// save
               save();
            }
            else if (n == JOptionPane.CANCEL_OPTION)
            {// cancel
                return false;
            }
         }
      }
      newFile = false;
      return true;
   }
   
   /**
    * Function checks whether there is inner frame of the same element we want 
    * to create. If not, creates new instance of Inner frame. Sets displayAsList
    * property, which is important for displaying one single element form a list
    * 
    * @param elem element to display in this frame
    * @param t title
    * @param parent parent iframe (to build tree-like structure)
    * @param displayAsList sets displayAsList
    * 
    * @return new instance of InnerFrame                            
    */       
   public InnerFrame createFrame(Element elem, String t, InnerFrame parent, boolean displayAsList)
   {
      // check whether frame exists
      JInternalFrame[] allFrames = desktop.getAllFrames();
      for (int i = 0; i < allFrames.length; i++)
      {
         if (((InnerFrame)allFrames[i]).getRootElement() == elem)
         {
            return null;
         }
      }
       
      InnerFrame frame = new InnerFrame(this, elem, t, parent, displayAsList);
      frame.setVisible(true);
      desktop.add(frame);
      return frame;
   }

   /**
    * Function checks whether there is inner frame of the same element we want 
    * to create. If not, creates new instance of Inner frame.
    * 
    * @param elem element to display in this frame
    * @param t title
    * @param parent parent iframe (to build tree-like structure)
    * 
    * @return new instance of InnerFrame                            
    */  
   public InnerFrame createFrame(Element elem, String t, InnerFrame parent)
   {
      // check whether frame exists
      JInternalFrame[] allFrames = desktop.getAllFrames();
      for (int i = 0; i < allFrames.length; i++)
      {
         if (((InnerFrame)allFrames[i]).getRootElement() == elem)
         {
            return null;
         }
      }
       
      InnerFrame frame = new InnerFrame(this, elem, t, parent, true);
      frame.setVisible(true); //necessary as of 1.3
      desktop.add(frame);
      return frame;
   }

   public Point findFreePlace()
   {
      // find the most bottom iframe
      JInternalFrame[] allFrames = desktop.getAllFrames();
      int y = 0;
      for (int i = 0; i < allFrames.length; i++)
      {
         if (allFrames[i].getY()+allFrames[i].getHeight() > y)
         {
            y = allFrames[i].getY()+allFrames[i].getHeight()+10;
         }
      }
      return new Point(10, y);
   }
   
   // Quit the application.
   protected void quit()
   {
      System.exit(0);
   }

   /**
    * File filter - filters XML files and directories
    */       
   private class XMLFileFilter extends javax.swing.filechooser.FileFilter
   {
      public boolean accept(File file)
      {
         return file.getName().toLowerCase().endsWith(".xml") || file.isDirectory();
      }
    
      public String getDescription()
      {
         return "XML Files ( *.xml )";
      }
   }

   /**
    *
    */       
   private File chooseFile(boolean open)
   {
      JFileChooser jfc = new JFileChooser();
      jfc.setCurrentDirectory(new File("."));
      jfc.setFileFilter(new XMLFileFilter());
      jfc.removeChoosableFileFilter(jfc.getAcceptAllFileFilter());
      int result;
      if (open)
      {
          result = jfc.showOpenDialog(null);
      }
      else
      {
          result = jfc.showSaveDialog(null);
      }
      if (result != JFileChooser.APPROVE_OPTION)
      {
         return null;
      }
      else
      { 
         return jfc.getSelectedFile();
      }
   }
    
   /**
    * @param args the command line arguments
    */
   public static void main(String args[])
   {
      java.awt.EventQueue.invokeLater(new Runnable()
      {
         public void run()
         {
            VisualXML v = new VisualXML("cs","CZ");
         }
      });
   }
   
   /**
    * Returns localized string from current localization bundle
    */       
   public String getString(String key)
   {
      return messages.getString(key);
   }

   /**
    * Sets edited
    */       
   public void setEdited(boolean b)
   {
      edited = b;
      resetTitle();
   }
   
   /**
    * Returns true if the opened document has been changed
    */       
   public boolean getEdited()
   {
      return edited;
   }
   
   /**
    * Changes title of the main JFrame 
    * Format: [Path to opened file (* if changed)] - AppName  
    */
   protected void resetTitle()
   {
      String t = new String("");
      if (xmlFile != null)
      {
         t = "[ "+xmlFile.getPath()+" ";
         if (edited) t = t + "*";
         t = t + "] - ";
      }
      else if(newFile)
      {
         t = "[ new document *] - ";
      }
      setTitle(t + getString("VisualXML.appname"));
   }
 
   /**
    * Updates content of all iframes
    */       
   public void updateIFrames(Element e)
   {
      JInternalFrame allFrames[] = desktop.getAllFrames();

      for (int i = 0; i < allFrames.length; i++)
      {
         if (allFrames[i] instanceof InnerFrame)
         {
            if (((InnerFrame)allFrames[i]).getRootElement() == e)
            {
               desktop.remove(allFrames[i]);
            }
            else
            {
               ((InnerFrame)allFrames[i]).updateContent();
            }
         }
      }
   }
}
