package src.xml;

import java.io.*;
import java.util.Iterator;

import java.util.List;
import java.util.ArrayList;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.XMLWriter;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.*;
import org.jaxen.*;

/**
 * Class parses given XML document and builds DOM tree
 */  
public class ProcessXML 
{
   private Document doc; // document
   private String path;  // path to xml file

   public ProcessXML()
   {
      doc = null;
   }
   
   public Document getDOMDoc()
   {
      return doc;
   }
   
   /**
    * Function reads and parses given file
    * @param path path to a xml document
    */
   public void createDOM(String path)
   {
      this.path = path;
      
      try
      {// reading and parsing XML file
         this.parseWithSAX(new File(path));
      }
      catch (DocumentException ex)
      {// error
         ex.printStackTrace();
         return;
      }
   }
  
   /**
    * Function saves XML document into a file
    * 
    * @param xmlFile target file
    * @param doc XML source
    * @throw a org.dom4j.IOException
    */       
   public static void saveXML(File xmlFile, Document doc) throws IOException
   {
      OutputFormat outformat = OutputFormat.createPrettyPrint();
      // change output encoding
      outformat.setEncoding(doc.getXMLEncoding());
              
      Writer out = new OutputStreamWriter(new FileOutputStream(xmlFile),doc.getXMLEncoding());
      XMLWriter writer = new XMLWriter(out, outformat);
      writer.write(doc);
      writer.flush();	
   }
  
   /**
    * Loads a document from xml file.
    *
    * @param f the xml source
    * @throw a org.dom4j.DocumentException occurs on parsing failure.
    */   
   public void parseWithSAX(File f) throws DocumentException
   {
      SAXReader xmlR = new SAXReader();
      doc = xmlR.read(f);
   }
}
